from .__about__ import (
    __title__, __summary__, __uri__, __version__, __author__, __license__, __copyright__,
)

__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__", "__license__", "__copyright__",
]
