__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__", "__license__", "__copyright__",
]

__title__ = "myautoml"
__summary__ = "myautoml is a package which provides a framework to automate machine learning"
__uri__ = "https://github.com/myautoml/myautoml"

__version__ = "0.2.4"

__author__ = "Erik Jan de Vries"

__license__ = ""
__copyright__ = "Copyright 2015-2020 {}".format(__author__)
